/*-----------------------------------------------------------------------------------------------------------*/
/*                                                ModelSwarm                                                 */
/*-----------------------------------------------------------------------------------------------------------*/

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;

import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

public class ModelSwarm extends SwarmImpl
{
    // Declare the Model parameter and their default values
    public int worldXSize = 10;
    public int worldYSize = 10;

    public double ProbDie = 0.25;

    public int nbSpecies = 2;

    public int endTime = 2000;

    Space mySpace;
    Species mySpecies;
    ScheduleImpl modelSchedule;


    /*-------------------------------------------------------------------------------*/
    /*                                     Constructor                               */
    /*-------------------------------------------------------------------------------*/

    public ModelSwarm(Zone aZone)
    {
	// Use the parent class
	super(aZone);
    }


    /*-------------------------------------------------------------------------------*/
    /*                                    BuildObjects                               */
    /*-------------------------------------------------------------------------------*/

    public Object buildObjects()
    {
	int xPos,yPos;

	// Use the parent class buildObject() to initialize the process
	super.buildObjects();

	// Create the model`s object
	mySpace = new Space(Globals.env.globalZone,worldXSize,worldYSize,ProbDie,nbSpecies);

	return this;
    }

    /*-------------------------------------------------------------------------------*/
    /*                                    BuildActions()                             */
    /*-------------------------------------------------------------------------------*/

    // Description : This is the method for building the list of actions for these objects
    //               to accomplish and for scheduling these actions in simulated time.

    public Object buildActions()
    {
	Selector sel;
	ActionGroupImpl modelActions;

	// Use the parent class to initialize the process
	super.buildActions();

	// Then create an ActionGroup object
	modelActions = new ActionGroupImpl(getZone());

	try
	{
	    sel = new Selector(mySpace.getClass(),"simulation",false);
	    modelActions.createActionTo$message(mySpace,sel);
	}
	catch (Exception e)
	{
	    System.err.println("Exception simulation: "+ e.getMessage());
	    System.exit(1);
	}

	// Now we add to ModelActions another message, this one to ModelSwarm itself
	// telling it to check the simulation time.
	try
	{
	    sel = new Selector(this.getClass(),"checkTime",false);
	    modelActions.createActionTo$message(this,sel);
	}
	catch (Exception e)
	{
	    System.err.println("Exception checkTime: "+ e.getMessage());
	    System.exit(1);
	}

	// Create the schedule
	modelSchedule = new ScheduleImpl(getZone(),1);

	modelSchedule.at$createAction(0,modelActions);

	return this;
    }

    /*------------------------------------------------------------------------------------------------*/
    /*                                            ActivateIn                                          */
    /*------------------------------------------------------------------------------------------------*/

    // Description : this method specifies the context in which the model is to be run.

    public Activity activateIn(Swarm swarmContext)
    {
	// Use the parent class to activate ourselves in the context passed to us
	super.activateIn(swarmContext);

	// then activate the schedule
	modelSchedule.activateIn(this);

	// We return the activity we have built
	return getActivity();

    }

    /*------------------------------------------------------------------------------------------------*/
    /*                                           CheckTime                                            */
    /*------------------------------------------------------------------------------------------------*/

    // Description : This is a method to end the simulation after an arbitrary number of periods given 
    //               by the endTime parameter.

    public void checkTime()
    {
	int i,t;
	int interval =5;

	if (Globals.env.getCurrentTime() >= endTime)
	{
	    // Terminate the simulation
	    System.out.println("We`ve reached our endTime at period "+ Globals.env.getCurrentTime());
	    Globals.env.getCurrentSwarmActivity().terminate();
	}
	else
	{
	    for (i = 0;(t = (int)Math.pow(interval,(double)i)) <= endTime;i++)
		if (t == Globals.env.getCurrentTime())
		    System.out.println("The time is " +t);

	}

	return;
    }

}

/*-----------------------------------------------------------------------------------------------------------*/
/*                                               End of ModelSwarm                                           */
/*-----------------------------------------------------------------------------------------------------------*/
