/*---------------------------------------------------------------------------------------------------------*/
/*                                                   Space.java                                            */
/*---------------------------------------------------------------------------------------------------------*/

// This class defines the area where species live

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.space.Discrete2dImpl;

public class Space extends Discrete2dImpl
{
    int worldXSize;              // Size X of the area
    int worldYSize;              // Size Y of the area

    double probDie;              // Probability to die in this area

    int numberSpecies;           // Number of different species
    int[] nbSpecies;             // Number of species for each different species

    int nbSeed;                 // Number of new seeds

    Species[][] ArraySpecies;    // Array of Species


    /*--------------------------------------------------------------------------------*/
    /*                                  Constructor                                   */
    /*--------------------------------------------------------------------------------*/

    public Space(Zone aZone,int wx,int wy,double probD,int nbSp)
    {
	// Call the constructor for the parent class
	super(aZone,wx,wy);

	int i,j,temp;

	// initialization
	worldXSize = wx;
	worldYSize = wy;

	probDie = probD;

	nbSeed = 0;

	numberSpecies = nbSp;
	nbSpecies = new int[numberSpecies];

	for (i = 0;i < numberSpecies;i++) nbSpecies[i] = 0;

	ArraySpecies = new Species[worldXSize][worldYSize];	

	for (i = 0;i < worldXSize;i++)
	    for (j = 0;j < worldYSize;j++)
	    {
		temp = Globals.env.uniformIntRand.getIntegerWithMin$withMax(1,numberSpecies);
		fastFillWithValue(temp);
		ArraySpecies[i][j] = new Species(i,j,++(nbSpecies[temp-1]),temp);
	    }
    
    }

    /*----------------------------------------------------------------------------------------*/
    /*                                        Death of Species                                */
    /*----------------------------------------------------------------------------------------*/

    // Description : this function determinates which species die.
    //               tempi and tempj are temporary variables to locate the death species

    public Point[] speciesDeath()
    {
	int i,tempi,tempj,t;
	Species sp;

	Point[] ArrayPoint;

	// initialization
	
	nbSeed = (int) (probDie*worldXSize*worldYSize);

	ArrayPoint = new Point[nbSeed];

	for (i = 0;i < nbSeed;i++)
	{
	    tempi = Globals.env.uniformIntRand.getIntegerWithMin$withMax(0,worldXSize);
	    tempj = Globals.env.uniformIntRand.getIntegerWithMin$withMax(0,worldYSize);

	    if (this.getValueAtX$Y(tempi,tempj) == 0) i--;
	    else 
	    {
		this.putValue$atX$Y(0,tempi,tempj);
		sp = ArraySpecies[tempi][tempj];
		t = sp.getType();
		nbSpecies[t-1]--;

		ArrayPoint[i] = new Point(tempi,tempj);

		ArraySpecies[tempi][tempj] = null;
	    }
	}

	return ArrayPoint;

    }

    /*-----------------------------------------------------------------------------------------*/
    /*                                            New seed                                     */
    /*-----------------------------------------------------------------------------------------*/

    // Description : function to create new seeds

    public int[] newSeed()
    {
	int[] ArrayNewSeed;
	int counter,t,i,j,k;           // i, j, k are variables forloop
	Species sp;

	counter = 0;
	for (i = 0;i < numberSpecies;i++) counter += nbSpecies[i];

	// initialization of ArrayNewSeed
	ArrayNewSeed = new int[counter];

	counter = 0;

	for (i = 0;i < worldXSize;i++)
	    for (j = 0;j < worldYSize;j++)
	    {
		if (this.getValueAtX$Y(i,j) != 0)
		{
		    sp = ArraySpecies[i][j];
		    t = sp.getType();
		    ArrayNewSeed[counter++] = t;
		} 
	    }

	return ArrayNewSeed;
    }

    /*---------------------------------------------------------------------------------------*/
    /*                                            AddSeed                                    */
    /*---------------------------------------------------------------------------------------*/

    // Description : method to add seed in the area
    //               nbSeed = number of seeds that we have to add

    public void addSeed(Point[] ArrayPoint,int[] ArraySeed)
    {
	int nbRandom;              // A random number
	int speciesALive;          // Number of species who are alive
	int seedType;              // Type of the new seed
	int i;                     // Variable for loop

	speciesALive = (worldXSize * worldYSize) - nbSeed;

	for (i = 1;i <= nbSeed;i++)
	{
	    nbRandom = Globals.env.uniformIntRand.getIntegerWithMin$withMax(0,speciesALive-1);

	    if (ArraySeed[nbRandom] == 0)
		i--;
	    else
	    {
		seedType = ArraySeed[nbRandom];
		ArraySeed[nbRandom] = 0;

		this.putValue$atX$Y(seedType,ArrayPoint[i-1].getX(),ArrayPoint[i-1].getY());
	    }
	}

    }

    /*---------------------------------------------------------------------------------------------*/
    /*                                             Simulation                                      */
    /*---------------------------------------------------------------------------------------------*/

    // Description : method to simulate the process

    public void simulation()
    {
	Point[] ArrayPoint;
	int [] ArrayNewSeed;
	int i;

	for (i = 0;i< 10;i++)
	{
	    ArrayPoint = this.speciesDeath();

	    ArrayNewSeed = this.newSeed();

	    this.addSeed(ArrayPoint,ArrayNewSeed);
	}
    }

}

/*---------------------------------------------------------------------------------------------------------*/
/*                                               End of Space                                              */
/*---------------------------------------------------------------------------------------------------------*/
